#!-PERL-
# File:		mif2mif.pl
# Version:	1.1
# Copyright:	MacDonald Dettwiler and Associates, 1994
# Author:	Duncan Fraser (djf@mda.ca)
# This file may be modified provided this header is kept intact.
# This code may not be included in a commercial product without
# the permission of the author.

# This script is part of the fm2html package.

# CHANGES since Version 1.0
#   - assumes initially that MIF file in a book has same name as document 
#     but with ".mif" appended; however, if file with that name is not
#     found, looks for name with ".doc" stripped, e.g. given "foo.doc" 
#     looks first for "foo.doc.mif", then for "foo.mif"
#   - by default, files are now modified in-place, e.g. "foo.mif" is moved
#     to "foo.mif.orig" (or "foo.mif.bak" if "foo.mif.orig" already exists)
#     and output is written to "foo.mif"; however, if "-e" option is used
#     to specify a filename extension then output is written to a new
#     filename, e.g. "foo.mif" is left untouched and output is written to
#     "foo.extension.mif"
#   - if "-e" is specified, cross-references are updated to point to new
#     filenames, e.g. to point to "foo.extension.doc" instead of "foo.doc"

$, = " ";		# set output field separator
$\ = "\n";		# set output record separator

$FALSE = 0;
$TRUE  = 1;

$MIF_MAGIC		= "<MIFFile";
$MIF_BOOK_MAGIC		= "<Book ";
$FM_MAGIC		= "<MakerFile";
$FM_BOOK_MAGIC		= "<Bookfile";

$PTAGS_EXTENSION	= "ptags";
$CTAGS_EXTENSION	= "ctags";

$TMPNAME		= "mif2mif.perl";

$GLOBAL_PTAGS_FILE	= "-LIBDIR-/ptags";
$GLOBAL_CTAGS_FILE	= "-LIBDIR-/ctags";

local( $extension, $ptags_file, $ctags_file, $infile, @mifs, %names,
       $base, $g_ptags, $g_ctags, $tmpfile, $file );

$tmpdir = $ENV{'TMPDIR'};
if ( ! $tmpdir ) { $tmpdir = "-TMPDIR-"; }
if ( ! $tmpdir ) { $tmpdir = `pwd`; chop( $tmpdir ); }
$tmpdir =~ s#/$##;
$tmpfile = "$tmpdir/$TMPNAME.$$";

while ( $ARGV[0] =~ m|^-| )
{
  $_ = shift;
  last if m|^--|;
  
  if ( m|^-e| )
  {
    $extension = $ARGV[0]; shift; next;
  }
  elsif ( m|^-c| )
  {
    $ctags_file = $ARGV[0]; shift; next;
  }
  elsif ( m|^-p| )
  {
    $ptags_file = $ARGV[0]; shift; next;
  }

  die "Invalid flag: \"$_\"";
}

if ( $#ARGV != 0 )
{
  print STDERR
    "USAGE: mif2mif [-p ptags_file] [-c ctags_file] [-e extension] mif-file";
  exit( 1 );
}

$infile = $ARGV[0];

if ( ! -e "$infile" )
  { die "mif2mif: Input file \"$infile\" does not exist... exiting"; }
if ( ! open( INFILE, "<$infile" ) )
  { die "mif2mif: Can't open input file \"$infile\"... exiting"; }

$magic = <INFILE>;

if ( $magic =~ m|^$FM_MAGIC|i || $magic =~ m|^$FM_BOOK_MAGIC|i )
  { die "mif2mif: FrameMaker file \"$infile\" is not a MIF file... exiting"; }
if ( $magic !~ m|^$MIF_MAGIC|i && $magic !~ m|^$MIF_BOOK_MAGIC|i )
  { die "mif2mif: Input file \"$infile\" is not a FrameMaker file... exiting";}

close( INFILE );

if ( $magic =~ m|^$MIF_BOOK_MAGIC|i )
{
  local( $inbook, $outbook );

  $inbook = "$infile";
  open( INBOOK, "<$inbook" )
    || die "mif2mif: Can't open input file \"$inbook\"... exiting";

  if ( "$extension" )
  {
    local( $suffix );

    $outbook = "$infile";
    $outbook =~ s|\.mif$||;
    $outbook =~ s|\.doc$|| ? ($suffix = ".doc") : ($suffix = "");
    $outbook = "$outbook$extension$suffix.mif";

    open( OUTBOOK, ">$outbook" )
      || die "mif2mif: Can't create new book file \"$outbook\"... exiting";

    print STDERR "mif2mif: Converting \"$inbook\" to \"$outbook\"...";
  }

  while ( <INBOOK> )
  {
    if ( m|([^<]*)<FileName[^`]*`<c\\>([^<]*)<U\\>([^']*)'>| )
    {
      local( $whitespace, $docname, $docpath, $mifpath );

      $whitespace = $1;
      $docname	  = $2;
      $docpath	  = $3;

      $mifpath = "$docpath.mif";

      if ( ! -e "$mifpath" )
      {
	local( $unstripped );
	$unstripped = $mifpath;
	$mifpath =~ s|\.doc||;

	if ( $mifpath eq $unstripped )
	{
	  print STDERR "mif2mif: ERROR: Input file \"$unstripped\" does not exist";
	  next;
	}
	elsif ( ! -e $mifpath )
	{
	  print STDERR "miftags: ERROR: Input file \"$unstripped\" does not exist, neither does \"$mifpath\"";
	  next;
	}
      }

      $mifs[++$#mifs] = $mifpath;

      if ( "$extension" )
      {
	local( $oldname, $suffix );

	$oldname = $docname;
	$docname =~ s|\.doc$|| ? ($suffix = ".doc") : ($suffix = "");
	$docname = "$docname$extension$suffix";
	$names{$oldname} = $docname;

	$docpath =~ s|\.doc$|| ? ($suffix = ".doc") : ($suffix = "");
	$docpath = "$docpath$extension$suffix";

	print OUTBOOK "$whitespace<FileName `<c\\>$docname<U\\>$docpath'>";
      }
    }
    elsif ( "$extension" )
    {
      chop; print OUTBOOK;
    }
  }

  close( INBOOK );
  if ( "$extension" )
    { close( OUTBOOK ); }

  @ARGV = @mifs;
}

$base = $infile;
$base =~ s|\.mif$||;
$ptags_file || ($ptags_file = "$base.ptags");

if ( ! -e $ptags_file )
{
  print STDERR "mif2mif: $ptags_file not found; copying global ptags file...";
  if ( -e $GLOBAL_PTAGS_FILE )
    { system( "cp $GLOBAL_PTAGS_FILE $ptags_file" ); }
  else
    { print STDERR "mif2mif: Global ptags file not found"; }
}
open( PTAGSF, "<$ptags_file" )
  || die "Can't open tags file \"$ptags_file\"";

$ctags_file || ($ctags_file = "$base.ctags");
if ( ! -e $ctags_file )
{
  print STDERR "mif2mif: $ctags_file not found; copying global ctags file...";
  if ( -e $GLOBAL_CTAGS_FILE )
    { system( "cp $GLOBAL_CTAGS_FILE $ctags_file" ); }
  else
    { print STDERR "mif2mif: Global ctags file not found"; }
}
open( CTAGSF, "<$ctags_file" )
  || die "Can't open tags file \"$ctags_file\"";

print STDERR "mif2mif: Using paragraph format tags from file \"$ptags_file\"";
print STDERR "mif2mif: Using character format tags from file \"$ctags_file\"";
if ( "$extension" )
  { print STDERR "mif2mif: Using output filename extension \"$extension\""; }

open( TMPF, ">$tmpfile" ) || die "Can't open temporary file \"$tmpfile\"";

print TMPF " \
#!/usr/local/bin/perl \
\
\$FALSE = 0; \
\$TRUE  = 1; \
\
sub substitute \
{ \
  local( \$inf, \$outf ) = \@_; \
  local( \$magic, $\ ); \
\
  \$\\ = \"\\n\"; \
\
  if ( ! -e \$inf ) \
  { \
    print STDERR \"mif2mif: ERROR: Can't convert \\\"\$inf\\\" (does not exist)\"; \
    return; \
  } \
\
  if ( ! open( INF, \"<\$inf\" ) ) \
  { \
    print STDERR \"mif2mif: ERROR: Can't open file \\\"\$inf\\\"\"; \
    return; \
  } \
\
  if ( ! open( OUTF, \">\$outf\" ) ) \
  { \
    print STDERR \"mif2mif: ERROR: Can't create file \\\"\$outf\\\"\"; \
    return; \
  } \
\
  \$magic = <INF>; \
\
  if ( \$magic !~ m|$MIF_MAGIC|i ) \
  { \
    print STDERR \"mif2mif: ERROR: File \\\"\$inf\\\" is not a MIF file\"; \
    return; \
  } \
\
  chop( \$magic ); print OUTF \$magic; \
\
  while ( <INF> ) \
  {";
  
while ( <PTAGSF> )
  { m|^([^\t]+)\t+([^\t\n]+)[\n\t]|o && ($1 ne $2) && ($g_ptags{$1} = $2); }
close( PTAGSF );

print TMPF " \
    if ( m|<PgfTag|o || m|<DeriveTag|o ) \
    {";

while ( ($from, $to) = each( %g_ptags ) )
{
  local( $escaped ); $escaped = &escape( $from );
  print TMPF "      s|`$escaped'|`$to'|o;";
}

print TMPF " \
    } \
    elsif ( m|<XRefSrcText|o || m|<MText|o ) \
    {";

while ( ($from, $to) = each( %g_ptags ) )
{
  local( $escaped ); $escaped = &escape( $from );
  print TMPF "      s|: $escaped:|: $to:|o;";
}

print TMPF " \
    } \
    elsif ( m|<XRefSrcFile|o ) \
    {";

while ( ($from, $to) = each( %names ) )
{
  local( $escaped ); $escaped = &escape( $from );
  print TMPF "      s|$escaped|$to|go;";
}

print TMPF " \
    } \
    elsif ( m|<VariableDef `<\\\$paranum\\[|o ) \
    {";

while ( ($from, $to) = each( %g_ptags ) )
  { print TMPF "      s|\\[$from\\]|[$to]|o;"; }

print TMPF "    }";

while ( <CTAGSF> )
  { m|^([^\t]+)\t+([^\t\n]+)[\n\t]|o && ($1 ne $2) && ($g_ctags{$1} = $2); }
close( CTAGSF );

if ( %g_ptags )
  { print TMPF "    elsif ( m|<FTag|o )"; }
else
  { print TMPF "    if ( m|<FTag|o )"; }

print TMPF "    {";

while ( ($from, $to) = each( %g_ctags ) )
  { print TMPF "      s|`$from'|`$to'|o;"; }

print TMPF "    }
\
    chop; print OUTF; \
  } \
\
  \$TRUE; \
}";

close( TMPF );

require( "$tmpfile" );

while ( $file = shift )
{
  local( $infile, $outfile );

  if ( "$extension" )
  {
    local( $suffix );

    $infile  = $file;

    $outfile = $file;
    $outfile =~ s|\.mif$||;
    $outfile =~ s|\.doc$|| && ($suffix = ".doc");
    $outfile = "$outfile$extension$suffix.mif";
  }
  else
  {
    $infile  = ( -e "$file.orig" ) ? "$file.bak" : "$file.orig"; 
    $outfile = $file;

    print STDERR "mif2mif: Renaming \"$file\" as \"$infile\"...";

    if ( ! rename( "$file", "$infile" ) )
    {
      print STDERR
	"mif2mif: ERROR: Can't convert \"$file\" (can't rename as \"$infile\")";
      next;
    }
  }

  print STDERR "mif2mif: Converting \"$infile\" to \"$outfile\"...";
  &substitute( $infile, $outfile );
}

system( "rm -f $tmpfile *.backup" );

print STDERR "mif2mif: Done";


sub escape	# take string, insert backslash before special characters
{
  local( $_ ) = @_;
  s|([\.\+\?\*\'\"\(\)])|\\$1|g;
  $_;
}
